from __future__ import annotations

import datetime as dt
from typing import TYPE_CHECKING
from zoneinfo import ZoneInfo

from hypothesis import given
from hypothesis.strategies import sampled_from

from utilities.tzdata import (
    NOW_HONG_KONG,
    NOW_TOKYO,
    TODAY_HONG_KONG,
    TODAY_TOKYO,
    HongKong,
    Tokyo,
    USCentral,
    USEastern,
    get_now_hong_kong,
    get_now_tokyo,
    get_today_hong_kong,
    get_today_tokyo,
)

if TYPE_CHECKING:
    from collections.abc import Callable


class TestGetNow:
    @given(case=sampled_from([(get_now_hong_kong, HongKong), (get_now_tokyo, Tokyo)]))
    def test_function(
        self, *, case: tuple[Callable[[], dt.datetime], ZoneInfo]
    ) -> None:
        func, time_zone = case
        self._assert(func(), time_zone)

    @given(case=sampled_from([(NOW_HONG_KONG, HongKong), (NOW_TOKYO, Tokyo)]))
    def test_constant(self, *, case: tuple[dt.datetime, ZoneInfo]) -> None:
        datetime, time_zone = case
        self._assert(datetime, time_zone)

    def _assert(self, datetime: dt.datetime, time_zone: ZoneInfo, /) -> None:
        assert isinstance(datetime, dt.datetime)
        assert datetime.tzinfo is time_zone


class TestGetToday:
    @given(func=sampled_from([get_today_hong_kong, get_today_tokyo]))
    def test_function(self, *, func: Callable[[], dt.date]) -> None:
        assert isinstance(func(), dt.date)

    @given(date=sampled_from([TODAY_HONG_KONG, TODAY_TOKYO]))
    def test_constant(self, *, date: dt.date) -> None:
        assert isinstance(date, dt.date)


class TestTimeZones:
    @given(time_zone=sampled_from([HongKong, Tokyo, USCentral, USEastern]))
    def test_main(self, *, time_zone: ZoneInfo) -> None:
        assert isinstance(time_zone, ZoneInfo)



from __future__ import annotations

import datetime as dt
from zoneinfo import ZoneInfo

from utilities.tzdata import HongKong, Tokyo
from utilities.tzlocal import (
    NOW_LOCAL,
    TODAY_LOCAL,
    get_local_time_zone,
    get_now_local,
    get_today_local,
)
from utilities.zoneinfo import UTC


class TestGetLocalTimeZone:
    def test_main(self) -> None:
        time_zone = get_local_time_zone()
        assert isinstance(time_zone, ZoneInfo)


class TestGetNowLocal:
    def test_function(self) -> None:
        self._assert(get_now_local())

    def test_constant(self) -> None:
        self._assert(NOW_LOCAL)

    def _assert(self, datetime: dt.datetime, /) -> None:
        assert isinstance(datetime, dt.datetime)
        ETC = ZoneInfo("Etc/UTC")  # noqa: N806
        assert datetime.tzinfo in {ETC, HongKong, Tokyo, UTC}


class TestGetTodayLocal:
    def test_function(self) -> None:
        assert isinstance(get_today_local(), dt.date)

    def test_constant(self) -> None:
        assert isinstance(TODAY_LOCAL, dt.date)



from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from types import TracebackType


class NoOpContextManager:
    def __enter__(self) -> None:
        return None

    def __exit__(
        self,
        exc_type: type[BaseException] | None,
        exc_val: BaseException | None,
        traceback: TracebackType | None,
    ) -> bool:
        _ = (exc_type, exc_val, traceback)
        return False


__all__ = ["NoOpContextManager"]



from __future__ import annotations

from typing import TYPE_CHECKING
from zoneinfo import ZoneInfo

from utilities.datetime import get_now, get_today

if TYPE_CHECKING:
    import datetime as dt


HongKong = ZoneInfo("Asia/Hong_Kong")
Tokyo = ZoneInfo("Asia/Tokyo")
USCentral = ZoneInfo("US/Central")
USEastern = ZoneInfo("US/Eastern")


def get_now_hong_kong() -> dt.datetime:
    """Get the current time in Hong Kong."""
    return get_now(time_zone=HongKong)


NOW_HONG_KONG = get_now_hong_kong()


def get_now_tokyo() -> dt.datetime:
    """Get the current time in Tokyo."""
    return get_now(time_zone=Tokyo)


NOW_TOKYO = get_now_tokyo()


def get_today_hong_kong() -> dt.date:
    """Get the current date in Hong Kong."""
    return get_today(time_zone=HongKong)


TODAY_HONG_KONG = get_today_hong_kong()


def get_today_tokyo() -> dt.date:
    """Get the current date in Tokyo."""
    return get_today(time_zone=Tokyo)


TODAY_TOKYO = get_today_tokyo()


__all__ = [
    "NOW_HONG_KONG",
    "NOW_TOKYO",
    "TODAY_HONG_KONG",
    "TODAY_TOKYO",
    "HongKong",
    "Tokyo",
    "USCentral",
    "USEastern",
    "get_now_hong_kong",
    "get_now_tokyo",
    "get_today_hong_kong",
    "get_today_tokyo",
]
